%% This is the code for the paper Structural Vector Autoregressions with Smooth Transition in Variances 
% The Interaction Between U.S. Monetary Policy and the Stock Market
% by Helmut Luetkepohl and Aleksei Netunajev
%% general part
clear
% uncomment next line to use parallel computing
%matlabpool open;
Nrep = 1000;                  % number of bootstrap replications
q = 0.84;                     % 68% confidence bands

%% VAR part
% obtain matrices of restrictions
[R_Matr_SR_BLeit, R_Matr_LR, R_Matr_SR_RC] = get_r_matr;

% estimate SVAR using Bjornland-Leitemo restrictions
VAR_BLeitemo =  classVAR(1, 1,  R_Matr_SR_BLeit,  R_Matr_LR);
VAR_BLeitemo = VAR_BLeitemo.estimateSVAR;

% bootstrap loop
for ii = 1: Nrep
    VAR_BLeitemo_BIRS(ii) = VAR_BLeitemo;
end
for ii = 1: Nrep
    disp(ii)
    VAR_BLeitemo_BIRS(ii) = VAR_BLeitemo_BIRS(ii).bootstr;
end
for ii = 1:Nrep
    BIRS_SR_BL(:,:, ii) = VAR_BLeitemo_BIRS(ii).IR_SRun;
    BIRS_LR_BL(:,:, ii) = VAR_BLeitemo_BIRS(ii).IR_LRun;
end
quantH_SR_BL = (quantile(BIRS_SR_BL,q, 3));
quantL_SR_BL = (quantile(BIRS_SR_BL,1-q, 3));
quantH_LR_BL = (quantile(BIRS_LR_BL,q, 3));
quantL_LR_BL = (quantile(BIRS_LR_BL,1-q, 3));
id = ' SVAR identified as in B-Leitemo';
plot_IR(VAR_BLeitemo.IR_SRun, VAR_BLeitemo.IR_LRun, quantH_SR_BL, quantL_SR_BL, quantH_LR_BL, quantL_LR_BL, VAR_BLeitemo.h, VAR_BLeitemo.T, id);

% estimate SVAR using recursive identification restrictions
VAR_RC =  classVAR(1, 0,  R_Matr_SR_RC,  []);
VAR_RC = VAR_RC.estimateSVAR;

for ii = 1: Nrep
    VAR_RC_BIRS(ii) = VAR_RC;
end
for ii = 1: Nrep
    disp(ii)
    VAR_RC_BIRS(ii) = VAR_RC_BIRS(ii).bootstr;
end
for ii = 1:Nrep
    BIRS_SR_RC(:,:, ii) = VAR_RC_BIRS(ii).IR_SRun;
    BIRS_LR_RC(:,:, ii) = VAR_RC_BIRS(ii).IR_LRun;
end
quantH_SR_RC = (quantile(BIRS_SR_RC,q, 3));
quantL_SR_RC = (quantile(BIRS_SR_RC,1-q, 3));
quantH_LR_RC = (quantile(BIRS_LR_RC,q, 3));
quantL_LR_RC = (quantile(BIRS_LR_RC,1-q, 3));
id = ' recursively identified SVAR';
plot_IR(VAR_RC.IR_SRun, VAR_RC.IR_LRun, quantH_SR_RC, quantL_SR_RC, quantH_LR_RC, quantL_LR_RC, VAR_RC.h, VAR_RC.T, id);

%% STAR part
gamma = -2.77;              % smoothness parameters
c = 167;                           % location of the transition
% vectorize the grid to pass it to parallel tasks
pairs = 1;
for i=1:numel(gamma)
    for j = 1:numel(c)
        g_pair(pairs, 1) = gamma(i);
        c_pair(pairs, 1) = c(j);
        pairs = pairs + 1;
    end
end
pairs = pairs - 1;

% initialize objects to optimize
for ii = 1 : pairs     
   STSVAR(ii) = classSTSVAR(g_pair(ii, 1), c_pair(ii, 1), 0, 0, [], []);
end

% perform the optimization of the STAR models on the grid
tic
for ii = 1: pairs
        disp(ii)
        STSVAR(ii) = STSVAR(ii).optimize;  
end
toc

% compute std.errors and IRs 
STSVAR = STSVAR.std_err;
STSVAR = STSVAR.IR;
STSVAR.plot_tr;

% bootstrap loop 
for jj = 1: Nrep
    BIRS(jj) = STSVAR;
end

for jj = 1: Nrep
    disp(jj)
    BIRS(jj) = BIRS(jj).bootstr;
end

% plot IRS
for ii = 1 :Nrep
    B_IR_SR(:,:, ii) = BIRS(ii).IR_SRun;
    B_IR_LR(:,:, ii) = BIRS(ii).IR_LRun;
end
q = 0.84;
quantH = (quantile(B_IR_SR,q, 3));
quantL = (quantile(B_IR_SR,1-q, 3));
quantH_LR = (quantile(B_IR_LR,q, 3));
quantL_LR = (quantile(B_IR_LR,1-q, 3));
id = ' ST-SVAR';
plot_IR(STSVAR.IR_SRun, STSVAR.IR_LRun, quantH, quantL, quantH_LR, quantL_LR, STSVAR.h, STSVAR.T, id);
%% Scale MP shock, plot joint figure for three different models, 
% Recursive
MP_Scaled_quantL(1 : 5, :) = quantL_SR_RC(21 : 25, :) ./ VAR_RC.IR_SRun(25,1);
MP_Scaled_quantH(1 : 5, :) = quantH_SR_RC(21 : 25, :) ./ VAR_RC.IR_SRun(25,1);
MP_Scaled_IR_SR(1:5, :) = VAR_RC.IR_SRun(21:25, :) ./ VAR_RC.IR_SRun(25,1);
% B-L identification
MP_Scaled_quantL(6 : 10, :) = quantL_SR_BL(21 : 25, :) ./ VAR_BLeitemo.IR_SRun(25,1);
MP_Scaled_quantH(6 : 10, :) = quantH_SR_BL(21 : 25, :) ./ VAR_BLeitemo.IR_SRun(25,1);
MP_Scaled_IR_SR(6 : 10, :) = VAR_BLeitemo.IR_SRun(21:25, :) ./ VAR_BLeitemo.IR_SRun(25,1);
%ST-SVAR
MP_Scaled_quantL(11 : 15, :) = quantL(1 : 5, :) ./ STSVAR.IR_SRun(5,1);
MP_Scaled_quantH(11 : 15, :) = quantH(1 : 5, :) ./ STSVAR.IR_SRun(5,1);
MP_Scaled_IR_SR(11 : 15, :) = STSVAR.IR_SRun(1:5, :) ./ STSVAR.IR_SRun(5,1);
% add the long run effect of the shock
MP_Scaled_quantL(4, :) = quantL_LR_RC(24, :) ./ VAR_RC.IR_SRun(25,1);
MP_Scaled_quantH(4, :) = quantH_LR_RC(24, :) ./ VAR_RC.IR_SRun(25,1);
MP_Scaled_IR_SR(4, :) = VAR_RC.IR_LRun(24, :) ./ VAR_RC.IR_SRun(25,1);

MP_Scaled_quantL(9, :) = quantL_LR_BL(24, :) ./ VAR_BLeitemo.IR_SRun(25,1);
MP_Scaled_quantH(9, :) = quantH_LR_BL(24, :) ./ VAR_BLeitemo.IR_SRun(25,1);
MP_Scaled_IR_SR(9, :) = VAR_BLeitemo.IR_LRun(24, :) ./ VAR_BLeitemo.IR_SRun(25,1);

MP_Scaled_quantL(14, :) = quantL_LR(4, :) ./ STSVAR.IR_SRun(5,1);
MP_Scaled_quantH(14, :) = quantH_LR(4, :) ./ STSVAR.IR_SRun(5,1);
MP_Scaled_IR_SR(14, :) = STSVAR.IR_LRun(4, :) ./ STSVAR.IR_SRun(5,1);

figure('Name','Monetary policy shocks','NumberTitle','off')
periods = 0 : 100;
Vec = reshape(reshape([1:15], 5 , 3)', 15, 1);
for p = 1 : 15
                subplot(5, 3, p);
                plot(periods, MP_Scaled_IR_SR(Vec(p),:),'k',  'LineWidth', 1.5);
                hold on;
                plot(periods, MP_Scaled_quantH(Vec(p),:),'k--',  'LineWidth', 1.5);
                plot(periods, MP_Scaled_quantL(Vec(p),:),'k--' ,  'LineWidth', 1.5);
                plot(periods, zeros(1, 100+1), 'k:' );
                hold off;
                xlim(gca, [0 100])   
end


%% Scale stock price shock    
% Recursive
SP_Scaled_quantL(1 : 5, :) = quantL_SR_RC(16 : 20, :) ./ VAR_RC.IR_SRun(19,1);
SP_Scaled_quantH(1 : 5, :) = quantH_SR_RC(16 : 20, :) ./ VAR_RC.IR_SRun(19,1);
SP_Scaled_IR_SR(1:5, :) = VAR_RC.IR_SRun(16:20, :) ./ VAR_RC.IR_SRun(19,1);
% B-L identification
SP_Scaled_quantL(6 : 10, :) = quantL_SR_BL(16 : 20, :) ./ VAR_BLeitemo.IR_SRun(19,1);
SP_Scaled_quantH(6 : 10, :) = quantH_SR_BL(16 : 20, :) ./ VAR_BLeitemo.IR_SRun(19,1);
SP_Scaled_IR_SR(6 : 10, :) = VAR_BLeitemo.IR_SRun(16:20, :) ./ VAR_BLeitemo.IR_SRun(19,1);
%ST-SVAR
SP_Scaled_quantL(11 : 15, :) = quantL(16 : 20, :) ./ STSVAR.IR_SRun(19,1);
SP_Scaled_quantH(11 : 15, :) = quantH(16 : 20, :) ./ STSVAR.IR_SRun(19,1);
SP_Scaled_IR_SR(11 : 15, :) = STSVAR.IR_SRun(16 : 20, :) ./ STSVAR.IR_SRun(19,1);
% add the long run effect of the shock
SP_Scaled_quantL(4, :) = quantL_LR_RC(19, :) ./ VAR_RC.IR_SRun(19,1);
SP_Scaled_quantH(4, :) = quantH_LR_RC(19, :) ./ VAR_RC.IR_SRun(19,1);
SP_Scaled_IR_SR(4, :) = VAR_RC.IR_LRun(19, :) ./ VAR_RC.IR_SRun(19,1);

SP_Scaled_quantL(9, :) = quantL_LR_BL(19, :) ./ VAR_BLeitemo.IR_SRun(19,1);
SP_Scaled_quantH(9, :) = quantH_LR_BL(19, :) ./ VAR_BLeitemo.IR_SRun(19,1);
SP_Scaled_IR_SR(9, :) = VAR_BLeitemo.IR_LRun(19, :) ./ VAR_BLeitemo.IR_SRun(19,1);

SP_Scaled_quantL(14, :) = quantL_LR(19, :) ./ STSVAR.IR_SRun(19,1);
SP_Scaled_quantH(14, :) = quantH_LR(19, :) ./ STSVAR.IR_SRun(19,1);
SP_Scaled_IR_SR(14, :) = STSVAR.IR_LRun(19, :) ./ STSVAR.IR_SRun(19,1);

figure('Name','Stock price shock','NumberTitle','off')
periods = 0 : 100;
for p = 1 : 15
                subplot(5, 3, p);
                plot(periods, SP_Scaled_IR_SR(Vec(p),:),'k',  'LineWidth', 1.5);
                hold on;
                plot(periods, SP_Scaled_quantH(Vec(p),:),'k--',  'LineWidth', 1.5);
                plot(periods, SP_Scaled_quantL(Vec(p),:),'k--' ,  'LineWidth', 1.5);
                plot(periods, zeros(1, 100+1), 'k:' );
                hold off;
                xlim(gca, [0 100])   
end